/**----------------------------------------------------------------------------
 * ImageMaker.cpp
 *-----------------------------------------------------------------------------
 * 
 *-----------------------------------------------------------------------------
 * All rights reserved by FSK Security Co.Ltd.
 *-----------------------------------------------------------------------------
 * 2011.3.18 created
**---------------------------------------------------------------------------*/ 
#include "stdafx.h"

#include <fstream>
#include <string>


#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#include <strsafe.h>

#include <stdio.h>
#include <conio.h>


/*  disk image layout

 0                    +512                  +512                  +512
 +---------------------+---------------------+---------------------+
 |    bootsector.bin   |   bootloader.bin    |  ntpayload.bin      |
 +---------------------+---------------------+---------------------+

*/

#define BOOT_SECTOR_NAME        L"bootsector.bin"
#define BOOT_LOADER_NAME        L"bootloader.bin"
#define NTPAYLOAD_NAME			L"ntpayload.bin"
#define BOOT_IMAGE_NAME         L"bootimage.img"

/**----------------------------------------------------------------------------
    \brief  
    
    \param  
    \return
    \code
    
    \endcode        
-----------------------------------------------------------------------------*/
int _tmain(int argc, _TCHAR* argv[])
{
    WCHAR current[MAX_PATH+1]={0};
    GetCurrentDirectory(MAX_PATH, current);

    WCHAR FileName[MAX_PATH+1]={0};
    
    // bootsector.bin  д´.
    // 
    StringCbPrintfW(FileName, sizeof(FileName), L"%s\\%s", current, BOOT_SECTOR_NAME);
    std::ifstream strm_bootsector(FileName, std::ios::in | std::ios::binary);
    if (TRUE != strm_bootsector.is_open())
    {
        fwprintf(stderr, L"can not open file=%s", BOOT_SECTOR_NAME);
        return -1;
    }
    fwprintf(stdout, L"[*] %s founded.\n", BOOT_SECTOR_NAME);

    // bootloader.bin  д´.
    // 
    StringCbPrintfW(FileName, sizeof(FileName), L"%s\\%s", current, BOOT_LOADER_NAME);
    std::ifstream strm_bootloader(FileName, std::ios::in | std::ios::binary);
    if (TRUE != strm_bootloader.is_open())
    {
        fwprintf(stderr, L"can not open file=%s", BOOT_LOADER_NAME);
        return -1;
    }
    fwprintf(stdout, L"[*] %s founded.\n", BOOT_LOADER_NAME);
    
	// ntpayload.bin  д´.
    // 
    StringCbPrintfW(FileName, sizeof(FileName), L"%s\\%s", current, NTPAYLOAD_NAME);
    std::ifstream strm_ntpayload(FileName, std::ios::in | std::ios::binary);
    if (TRUE != strm_ntpayload.is_open())
    {
        fwprintf(stderr, L"can not open file=%s", NTPAYLOAD_NAME);
        return -1;
    }
    fwprintf(stdout, L"[*] %s founded.\n", NTPAYLOAD_NAME);
    

	

    // boot.img  Ѵ.
    //
    StringCbPrintfW(FileName, sizeof(FileName), L"%s\\%s", current, BOOT_IMAGE_NAME);
    std::ofstream strm_image(FileName, std::ios::out| std::ios::trunc | std::ios::binary);
    if (TRUE != strm_image.is_open())
    {
        fwprintf(stderr, L"can not open file=%s", BOOT_IMAGE_NAME);
        return -1;
    }
    fwprintf(stdout, L"\n[*] %s created.\n", BOOT_IMAGE_NAME);
    

    // diskimage   0xcc  ä ̹ Ѵ.
	// 
    DWORD CurrentPosition=0;
    char tmp[512 * 3]; RtlFillMemory(tmp, sizeof(tmp), 0xcc);    
    strm_image.seekp(0, std::ios::beg);
    strm_image.write(tmp, sizeof(tmp));
	
	
	
	// 
    // bootsector.bin  ù° (0~511bytes)  . 
    // 
    strm_image.seekp(0, std::ios::beg);

    strm_bootsector.seekg(0, std::ios::end);
    int size = (int) strm_bootsector.tellg();
    if (512 < size ) 
    {
        fwprintf(stderr, L"%s file size is invalid, size=%u bytes", BOOT_SECTOR_NAME, size);
        return -1;
    }
    strm_bootsector.seekg(0, std::ios::beg);
    if(!strm_bootsector.read(tmp, size))
    {
        fwprintf(stderr, L"can not read from file=%s", BOOT_SECTOR_NAME);
        return -1;
    }
    
    strm_bootsector.seekg(0, std::ios::beg);
    if (!strm_image.write(tmp, size))
    {
        fwprintf(stderr, L"can not write %s to %s", BOOT_SECTOR_NAME, BOOT_SECTOR_NAME);
        return -1;    
    }
    
    CurrentPosition = 512;
    
    //
    // bootloader.bin  ι° (512~1023bytes)  .
    //
    RtlFillMemory(tmp, sizeof(tmp), 0xcc);

    strm_bootloader.seekg(0, std::ios::end);
    size = (int) strm_bootloader.tellg();
    if (512 < size)
    {
        fwprintf(stderr, L"%s file size is invalid, size=%u bytes", BOOT_LOADER_NAME, size);
        return -1;
    }

    strm_bootloader.seekg(0, std::ios::beg);
    if (!strm_bootloader.read(tmp, size))
    {
        fwprintf(stderr, L"can not read from file=%s", BOOT_LOADER_NAME);
        return -1;
    }

    strm_image.seekp(CurrentPosition, std::ios::beg);
    if (!strm_image.write(tmp, size))
    {
        fwprintf(stderr, L"can not write %s to %s", BOOT_LOADER_NAME, BOOT_IMAGE_NAME);
        return -1;    
    }


    CurrentPosition += 512;
    
    //
    // ntpayload.bin  ι° (1024~1536 bytes)  .
    //
    RtlFillMemory(tmp, sizeof(tmp), 0xcc);

    strm_ntpayload.seekg(0, std::ios::end);
    size = (int) strm_ntpayload.tellg();
    if (512 < size)
    {
        fwprintf(stderr, L"%s file size is invalid, size=%u bytes", NTPAYLOAD_NAME, size);
        return -1;
    }

    strm_ntpayload.seekg(0, std::ios::beg);
    if (!strm_ntpayload.read(tmp, size))
    {
        fwprintf(stderr, L"can not read from file=%s", NTPAYLOAD_NAME);
        return -1;
    }

    strm_image.seekp(CurrentPosition, std::ios::beg);
    if (!strm_image.write(tmp, size))
    {
        fwprintf(stderr, L"can not write %s to %s", NTPAYLOAD_NAME, BOOT_IMAGE_NAME);
        return -1;    
    }
	

    strm_bootloader.close();
    strm_bootsector.close();
	strm_ntpayload.close();
    strm_image.close();

    fwprintf(stdout, L"[*] boot image = %s created successfully.\n", BOOT_IMAGE_NAME);
    fwprintf(stdout, L"  \npress enter...");
    getch();
}


